/** BEGIN COPYRIGHT BLOCK
 * This Program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; version 2 of the License.
 *
 * This Program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this Program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 * In addition, as a special exception, Red Hat, Inc. gives You the additional
 * right to link the code of this Program with code not covered under the GNU
 * General Public License ("Non-GPL Code") and to distribute linked combinations
 * including the two, subject to the limitations in this paragraph. Non-GPL Code
 * permitted under this exception must only link to the code of this Program
 * through those well defined interfaces identified in the file named EXCEPTION
 * found in the source code files (the "Approved Interfaces"). The files of
 * Non-GPL Code may instantiate templates or use macros or inline functions from
 * the Approved Interfaces without causing the resulting work to be covered by
 * the GNU General Public License. Only Red Hat, Inc. may make changes or
 * additions to the list of Approved Interfaces. You must obey the GNU General
 * Public License in all respects for all of the Program code and other code used
 * in conjunction with the Program except the Non-GPL Code covered by this
 * exception. If you modify this file, you may extend this exception to your
 * version of the file, but you are not obligated to do so. If you do not wish to
 * provide this exception without modification, you must delete this exception
 * statement from your version and license this file solely under the GPL without
 * exception.
 *
 *
 * Copyright (C) 2013 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#ifndef LIBOTP_H_
#define LIBOTP_H_

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <dirsrv/slapi-plugin.h>
#include <stdbool.h>
#include <stdlib.h>

struct otptoken;

/* Frees the token array. */
void otptoken_free_array(struct otptoken **tokens);

/* Find tokens.
 *
 * All criteria below are cumulative. For example, if you specify both dn and
 * active and the token at the dn specified isn't active, an empty array will
 * be returned.
 *
 * If owner is not NULL, the owner's tokens are returned.
 *
 * If dn is not NULL, only this specified token is returned.
 *
 * If active is true, only tokens that are active are returned.
 *
 * If filter is not NULL, the filter will be added to the search criteria.
 *
 * Returns NULL on error. If no tokens are found, an empty array is returned.
 * The array is NULL terminated.
 */
struct otptoken **otptoken_find(Slapi_ComponentId *id, Slapi_DN *owner,
                                Slapi_DN *dn, bool active, const char *filter);

/* Get the length of the token code. */
int otptoken_get_digits(struct otptoken *token);

/* Get the SDN of the token. */
const Slapi_DN *otptoken_get_sdn(struct otptoken *token);

/* Validate the token code within a range of steps. */
bool otptoken_validate(struct otptoken *token, size_t steps, uint32_t code);

/* Validate the token code within a range of steps. If tail is true,
 * it will be assumed that the token is specified at the end of the string. */
bool otptoken_validate_string(struct otptoken *token, size_t steps,
                       const char *code, ssize_t len, bool tail);

/* Synchronize the token within a range of steps. */
bool otptoken_sync(struct otptoken * const *tokens, size_t steps,
                   uint32_t first_code, uint32_t second_code);

#endif /* LIBOTP_H_ */
